import React, { useState, useEffect } from 'react';
import { render } from 'react-dom';
import throttle from 'lodash-es/throttle';

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

// A type of promise-like that resolves synchronously and supports only one observer
var _Pact = /*#__PURE__*/function () {
  function _Pact() {}

  _Pact.prototype.then = function (onFulfilled, onRejected) {
    var result = new _Pact();
    var state = this.s;

    if (state) {
      var callback = state & 1 ? onFulfilled : onRejected;

      if (callback) {
        try {
          _settle(result, 1, callback(this.v));
        } catch (e) {
          _settle(result, 2, e);
        }

        return result;
      } else {
        return this;
      }
    }

    this.o = function (_this) {
      try {
        var value = _this.v;

        if (_this.s & 1) {
          _settle(result, 1, onFulfilled ? onFulfilled(value) : value);
        } else if (onRejected) {
          _settle(result, 1, onRejected(value));
        } else {
          _settle(result, 2, value);
        }
      } catch (e) {
        _settle(result, 2, e);
      }
    };

    return result;
  };

  return _Pact;
}(); // Settles a pact synchronously

function _settle(pact, state, value) {
  if (!pact.s) {
    if (value instanceof _Pact) {
      if (value.s) {
        if (state & 1) {
          state = value.s;
        }

        value = value.v;
      } else {
        value.o = _settle.bind(null, pact, state);
        return;
      }
    }

    if (value && value.then) {
      value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
      return;
    }

    pact.s = state;
    pact.v = value;
    var observer = pact.o;

    if (observer) {
      observer(pact);
    }
  }
}
function _isSettledPact(thenable) {
  return thenable instanceof _Pact && thenable.s & 1;
} // Converts argument to a function that always returns a Promise
var _iteratorSymbol = /*#__PURE__*/typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator = /*#__PURE__*/Symbol("Symbol.iterator")) : "@@iterator"; // Asynchronously iterate through an object's values
var _asyncIteratorSymbol = /*#__PURE__*/typeof Symbol !== "undefined" ? Symbol.asyncIterator || (Symbol.asyncIterator = /*#__PURE__*/Symbol("Symbol.asyncIterator")) : "@@asyncIterator"; // Asynchronously iterate on a value using it's async iterator if present, or its synchronous iterator if missing

function _for(test, update, body) {
  var stage;

  for (;;) {
    var shouldContinue = test();

    if (_isSettledPact(shouldContinue)) {
      shouldContinue = shouldContinue.v;
    }

    if (!shouldContinue) {
      return result;
    }

    if (shouldContinue.then) {
      stage = 0;
      break;
    }

    var result = body();

    if (result && result.then) {
      if (_isSettledPact(result)) {
        result = result.s;
      } else {
        stage = 1;
        break;
      }
    }

    if (update) {
      var updateValue = update();

      if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
        stage = 2;
        break;
      }
    }
  }

  var pact = new _Pact();

  var reject = _settle.bind(null, pact, 2);

  (stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
  return pact;

  function _resumeAfterBody(value) {
    result = value;

    do {
      if (update) {
        updateValue = update();

        if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
          updateValue.then(_resumeAfterUpdate).then(void 0, reject);
          return;
        }
      }

      shouldContinue = test();

      if (!shouldContinue || _isSettledPact(shouldContinue) && !shouldContinue.v) {
        _settle(pact, 1, result);

        return;
      }

      if (shouldContinue.then) {
        shouldContinue.then(_resumeAfterTest).then(void 0, reject);
        return;
      }

      result = body();

      if (_isSettledPact(result)) {
        result = result.v;
      }
    } while (!result || !result.then);

    result.then(_resumeAfterBody).then(void 0, reject);
  }

  function _resumeAfterTest(shouldContinue) {
    if (shouldContinue) {
      result = body();

      if (result && result.then) {
        result.then(_resumeAfterBody).then(void 0, reject);
      } else {
        _resumeAfterBody(result);
      }
    } else {
      _settle(pact, 1, result);
    }
  }

  function _resumeAfterUpdate() {
    if (shouldContinue = test()) {
      if (shouldContinue.then) {
        shouldContinue.then(_resumeAfterTest).then(void 0, reject);
      } else {
        _resumeAfterTest(shouldContinue);
      }
    } else {
      _settle(pact, 1, result);
    }
  }
} // Asynchronously implement a do ... while loop

var Highlighter = /*#__PURE__*/function () {
  function Highlighter(getTextNodes, match, minBatchTextLength) {
    if (minBatchTextLength === void 0) {
      minBatchTextLength = 1000;
    }

    this.getTextNodes = getTextNodes;
    this.match = match;
    this.minBatchTextLength = minBatchTextLength;
    this.matches = [];
  }

  var _proto = Highlighter.prototype;

  _proto.scan = function scan() {
    try {
      var _this2 = this;

      _this2.matches.forEach(function (m) {
        return m.ranges.forEach(function (r) {
          return r.detach();
        });
      });

      _this2.matches = [];
      return Promise.resolve(_this2.getTextNodes()).then(function (nodes) {
        function _temp3() {
          _this2.updateHighlights();
        }

        var l = nodes.length;
        var batch = [];
        var batchTextLength = 0;
        var i = 0;

        var _temp2 = _for(function () {
          return i < l;
        }, function () {
          return !!(i += 1);
        }, function () {
          var node = nodes[i];
          batch.push(node);
          batchTextLength += (node.nodeValue || '').length;

          var _temp = function () {
            if (i === l - 1 || batchTextLength > _this2.minBatchTextLength) {
              return Promise.resolve(_this2.match(batch.map(function (n) {
                return n.nodeValue || '';
              }))).then(function (alertGroups) {
                batch.forEach(function (n, i) {
                  _this2.matches.push({
                    node: n,
                    alerts: alertGroups[i],
                    ranges: []
                  });
                });
                batch = [];
                batchTextLength = 0;
              });
            }
          }();

          if (_temp && _temp.then) return _temp.then(function () {});
        });

        return _temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.updateHighlights = function updateHighlights() {
    var _loop = function _loop() {
      var m = _step.value;
      m.ranges.forEach(function (r) {
        return r.detach();
      });
      m.ranges = m.alerts.map(function (a) {
        var r = document.createRange();
        r.setStart(m.node, a.start);
        r.setEnd(m.node, a.end);
        return r;
      });
    };

    for (var _iterator = _createForOfIteratorHelperLoose(this.matches), _step; !(_step = _iterator()).done;) {
      _loop();
    }
  };

  return Highlighter;
}();

var Highlights = function Highlights(_ref) {
  var highlighter = _ref.highlighter;

  var _useState = useState(0),
      count = _useState[0],
      setCount = _useState[1];

  useEffect(function () {
    // TODO: mutation observer, re-scan on mutations.
    highlighter.scan().then(function () {
      return setCount(function (v) {
        return v + 1;
      });
    });
    var update = throttle(function () {
      console.log('Updating highlights...');
      highlighter.updateHighlights();
      setCount(function (v) {
        return v + 1;
      });
    }, 500);
    document.addEventListener('scroll', update);
    window.addEventListener('resize', update);
  }, [highlighter, setCount]);
  return React.createElement("div", null, React.createElement("strong", null, "Count: ", count), highlighter.matches.map(function (m) {
    return m.ranges.filter(Boolean).map(function (r, index) {
      var alert = m.alerts[index];
      return Array.from(r.getClientRects()).map(function (rect, index) {
        return React.createElement("div", {
          key: alert.id + index,
          className: "dh-underline",
          style: {
            zIndex: 999999,
            position: 'fixed',
            top: rect.top + rect.height - 1,
            left: rect.left,
            width: rect.width,
            height: 2,
            background: alert.color
          }
        }, React.createElement("div", {
          style: {
            background: alert.color,
            color: '#ffffff'
          }
        }, React.createElement("ul", null, React.createElement("li", null, "Card ID: ", alert.id), React.createElement("li", null, "Title: ", alert.keyword))));
      });
    });
  }).flat(3));
};

/**
 * Tells whether an HTMLElement is visible.
 * @remarks
 * - An HTMLElement is visible doesn't mean people can see it on the screen.
 * - The implementation is based on `jQuery`'s `:visible` selector. See
 *   https://github.com/jquery/jquery/blob/d0ce00cdfa680f1f0c38460bc51ea14079ae8b07/src/css/hiddenVisibleSelectors.js
 * - Alternatively, if not considering `position: fixed`, use `e.offsetParent !== null`. See
 *   https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement/offsetParent
 * @param e The HTMLElement to test.
 * @returns Returns `true` if the HTMLElement is visible as defined by CSS rules;
 * otherwise `false`.
 */
var isElementVisible = function isElementVisible(e) {
  return !!(e.offsetWidth || e.offsetHeight || e.getClientRects().length);
};

/**
 * Tells whether the given DOMRect intersects with the viewport.
 * @param param0 The DOMRect to test.
 * @returns Returns `true` if any pixel in the DOMRect is visible on the screen,
 * even it is under the scrollbars; otherwise `false`;
 */
var isRectIntersectViewport = function isRectIntersectViewport(_ref) {
  var left = _ref.left,
      right = _ref.right,
      top = _ref.top,
      bottom = _ref.bottom;
  var vw = window.innerWidth || document.documentElement.clientWidth;
  var vh = window.innerHeight || document.documentElement.clientHeight;
  return top >= 0 && top <= vh && left >= 0 && left <= vw || // top left
  bottom >= 0 && bottom <= vh && left >= 0 && left <= vw || // bottom left
  top >= 0 && top <= vh && right >= 0 && right <= vw || // top right
  bottom >= 0 && bottom <= vh && right >= 0 && right <= vw // bottom right
  ;
};

// eslint-disable-next-line no-control-regex
var REGEX_TRIM_CONTROL_CHARS = /^[\u0000-\u001F\u21B5\s]+|[\u0000-\u001F\u21B5\s]+$/gm;
/**
 * Trim both leading and trailing control characters from every line.
 * @remarks
 *  New line characters (`\r` and `\n`) are left untouched.
 * @param s The string to trim.
 * @returns A new string with both leading and trailing control characters deleted from every line.
 */

var trimInvisibleChars = function trimInvisibleChars(s) {
  return s.replace(REGEX_TRIM_CONTROL_CHARS, '');
};

/**
 * Gets all text nodes on the page.
 * @param visibleOnly Exclude invisible text nodes.
 * @param trimInvisibleChars Whether to delete both leading and
 * trailing invisible characters before counting node text length.
 * @param minLength Exclude text nodes whose text length is less than give value.
 * @returns An array of text nodes that satisfy given conditions.
 */

var getTextNodes = function getTextNodes(visibleOnly, trimInvisibleChars$1, minLength) {
  if (visibleOnly === void 0) {
    visibleOnly = true;
  }

  if (trimInvisibleChars$1 === void 0) {
    trimInvisibleChars$1 = true;
  }

  if (minLength === void 0) {
    minLength = 3;
  }

  var walker = document.createTreeWalker(document.documentElement, NodeFilter.SHOW_TEXT, null, false);
  var nodes = [];

  while (walker.nextNode()) {
    var node = walker.currentNode;
    var text = node.nodeValue || '';

    if (minLength > 0) {
      var s = trimInvisibleChars$1 ? trimInvisibleChars(text) : text;

      if (s.length < minLength) {
        continue;
      }
    }

    if (visibleOnly) {
      var parentElement = node.parentElement;

      if (!parentElement || !isElementVisible(parentElement)) {
        continue;
      }

      if (!isRectIntersectViewport(parentElement.getBoundingClientRect())) {
        continue;
      }
    }

    nodes.push(node);
  }

  return nodes;
};

var start = function start(match, minBatchTextLength) {
  if (minBatchTextLength === void 0) {
    minBatchTextLength = 1000;
  }

  var highlighter = new Highlighter(getTextNodes, match, minBatchTextLength);
  var highlights = document.createElement('highlights');
  document.body.appendChild(highlights);
  render(React.createElement(Highlights, {
    highlighter: highlighter
  }), highlights);
};

export { Highlighter, getTextNodes, isElementVisible, isRectIntersectViewport, start, trimInvisibleChars };
//# sourceMappingURL=dom-highlight-lib.esm.js.map
